@@ -48,8 +48,26 @@ def get_channel(e):
 	return int((e-1.6)/0.04)
 
 def create_user_gti(time, in_rate, in_drate, lognormal=True, gti_name='user_gtis.fits', 
-                    level_down=100, level_up = 0, quantile=0.8, make_plot=True, 
-                    far=1e-4):
+                    in_level_down=0, in_level_up = 1e10, quantile=0.8, make_plot=True, 
+                    far=1e-4, histo_bins=300):
+	"""Creates user GTI based on hisogram of rates
+
+	Args:
+		time (numpy array): time array
+		in_rate (numpy array): input rate
+		in_drate (numpy array): input rate uncertainty
+		lognormal (bool, optional): if using a lognorml distribution. Defaults to True.
+		gti_name (str, optional): GTI file name to be generated. Defaults to 'user_gtis.fits'.
+		level_down (int, optional): not lower than this. Defaults to 0. 
+		level_up (int, optional): not larger than this. Defaults to 1e10.
+		quantile (float, optional): quantile to fit the distribution. Defaults to 0.8.
+		make_plot (bool, optional): if making plots. Defaults to True.
+		far (_type_, optional): the False Alarm Rate to exclude intervals from the fitted distribution. Defaults to 1e-4.
+		histo_bins (int, optional): number of bins in the histogram. Defaults to 300.
+
+	Returns:
+		_type_: _description_
+	"""	
 	import numpy as np
 	from stingray.gti import create_gti_from_condition
 	from astropy.io import fits as pf
@@ -59,18 +77,21 @@ def create_user_gti(time, in_rate, in_drate, lognormal=True, gti_name='user_gtis
 		rate = np.log10(in_rate)
 		drate = in_drate/in_rate
 		rate_label = '$\\log_{10}(\\mathrm{rate})$'
+		level_down = np.log10(max(level_down, 1e-5))
+		level_up = np.log10(max(level_up, 1e-5))		
 	else:
 		rate = in_rate
 		drate = in_drate
 		rate_label = 'rate'
-    
+		level_down = in_level_down
+		level_up = in_level_up
 
 	limit_down, limit_up = utils.rate_filter(time, rate, drate, 
                                        level_down=level_down, 
                                        level_up = level_up, 
                                        quantile=quantile, 
                                        make_plot=make_plot, 
-                                       far=far, rate_label=rate_label)
+                                       far=far, rate_label=rate_label, histo_bins=histo_bins)
     
 	new_gti = create_gti_from_condition(time, (rate > limit_down) & (rate < limit_up) )
 
@@ -515,14 +536,15 @@ def wrap_process(obsid, ra_src, dec_src, outdir_base, pipeline_flag=False, regio
 		
 		os.chdir(outdir_pipeline)
 		for unit in ['A', 'B']:
-			with open('xsel_%s.txt' % unit, 'w') as ff:
-				ff.write(xsel_cmd % (obsid, unit, chan_min, chan_max, unit))
-			cmd = 'xselect < ' + 'xsel_%s.txt' % (unit)
-			run(cmd)
-		os.chdir('..')
-
+			if not os.path.isfile('ima%s.fits' % unit):
+				with open('xsel_%s.txt' % unit, 'w') as ff:
+					ff.write(xsel_cmd % (obsid, unit, chan_min, chan_max, unit))
+				cmd = 'xselect < ' + 'xsel_%s.txt' % (unit)
+				run(cmd)
+		
 		raA, decA, r_max_arcsecA, ra_backA, dec_backA = prepare_region('A')
 		raB, decB, r_max_arcsecB, ra_backB, dec_backB = prepare_region('B')
+		os.chdir('..')
 
 		logger.info("We store regions in the spectral extraction folder '%s'" % outdir)
 
@@ -542,10 +564,10 @@ def wrap_process(obsid, ra_src, dec_src, outdir_base, pipeline_flag=False, regio
 
 		if no_ds9_flag==False:
 			logger.info("Please check the regions for FPMA")
-			cmd = "ds9 -scale log " + outdir_pipeline + "/nu" + obsid + "A01_cl.evt -regions " + src_regionA + " -regions " + bkg_regionA
+			cmd = "ds9 -scale log " + outdir_pipeline + "/imaA.fits -regions " + src_regionA + " -regions " + bkg_regionA
 			run(cmd)
 			logger.info("Please check the regions for FPMB")
-			cmd = "ds9 -scale log " + outdir_pipeline + "/nu" + obsid + "B01_cl.evt -regions " + src_regionB + " -regions " + bkg_regionB
+			cmd = "ds9 -scale log " + outdir_pipeline + "/imaB.fits -regions " + src_regionB + " -regions " + bkg_regionB
 			run(cmd)
 
 			#logger.info("Have you checked properly the region files?(y/Y will continue the processing any other key stop it)")
