@@ -4,6 +4,7 @@ import copy
 import numpy as np
 import matplotlib.pyplot as plt
 import os
+import shutil
 
 # file_handler = logging.FileHandler(filename='nustar_utils_%s.log' % (strftime("%Y-%m-%dT%H:%M:%S", gmtime())))
 # stdout_handler = logging.StreamHandler(sys.stdout)
@@ -175,6 +176,53 @@ def get_efold_frequency(nu_min, nu_max, min_en=3., max_en=20., n_bins=32, unit='
 
     return x[2]
 
+def get_nu_search_limits(filename, freq_search_interval=0.2, plot_filename=None, nyquist_factor=1, min_frequency=1e-4):
+    """It uses the Lomb-Scargle periodogram to find a range of spin frequencies to explore
+        to find a reliable periodicity
+
+    Args:
+        filename (_type_): name of the light curve in fits format to load
+        freq_search_interval (float, optional): it will provide a search interval from (1-/+freq_search_interval) of the peak. Defaults to 0.2.
+        plot_filename (str, optional): the name of output plot file. Defaults to None to avoid plotting.
+        nyquist_factor (int, optional): Lomb Scargle nymquist factor, see https://docs.astropy.org/en/stable/timeseries/lombscargle.html. Defaults to 1.
+        min_frequency (float, optional): it will search peaks only above this frequency (to avoid red-noise). Defaults to 1e-4
+    Returns:
+        nu_start, nu_stop: minimum and maximum frequencies to search
+    """    
+
+    import hratio
+    lc_unbinned = hratio.light_curve()
+    lc_unbinned.read_from_fits(filename=filename)
+    from astropy.timeseries import LombScargle
+    max_freq = 0.5/lc_unbinned._timedel_
+    min_freq = max(1e-4,1./(lc_unbinned._tstop_ - lc_unbinned._tstart_ ))
+    #frequencies = np.arange(min_freq, max_freq, min_freq)
+    #power = LombScargle(lc_unbinned._times_, lc_unbinned._rates_).power(frequencies)
+    frequencies, power = LombScargle(lc_unbinned._times_, lc_unbinned._rates_).autopower(nyquist_factor=nyquist_factor)
+    
+    mask = frequencies > min_frequency
+    max_freq = frequencies[mask][np.argmax(power[mask])]
+    nu_start = max_freq * (1.-freq_search_interval)
+    nu_stop = max_freq * (1.+freq_search_interval)
+    logger.info("We suggest to explore from %f to %f Hz" %(nu_start, nu_stop))
+    
+    if plot_filename is not None:
+        plt.figure()
+        plt.loglog(frequencies, power) 
+        plt.xlabel('Frequency [Hz]')
+        plt.ylabel('Power (Lomb-Scargle)')
+        plt.axvspan(nu_start, nu_stop, alpha=0.5, color='cyan')
+        plt.savefig(plot_filename)
+
+    return nu_start, nu_stop
+    
+
+    
+    
+
+
+
+
 enphase_cmd='''17
 list_evt.txt
 none
@@ -259,6 +307,7 @@ y
 
 
 def make_enphase(freq,  min_en=3., max_en=70., en_step=0.5, n_bins=32, orbitfile=None, nudot=0):
+    
     '''
     Wrapper on timingsuite to produce Energy - Phase matrixes for source and background for both FPM units.
     :param freq: spin frequency
@@ -298,10 +347,13 @@ def make_enphase(freq,  min_en=3., max_en=70., en_step=0.5, n_bins=32, orbitfile
                     else:
                         if not os.path.isfile(en_step):
                             raise FileExistsError('File %s does not exist' % en_step)
-                        ff.write(enphase_cmd_orbit_binfile % ('%s%s_ENPHASE.fits' % (tt, unit), orbitfile,
-                                                      n_bins, freq, nudot, en_step))
-
+                        ff.write(enphase_cmd_orbit_binfile % ('%s%s_ENPHASE.fits' % (tt, unit), 
+                        orbitfile, n_bins, freq, nudot, en_step))
+            
             run()
+            shutil.copy('%s%s_ENPHASE.fits' % (tt, unit), '%s%s_ENPHASE_%03d.fits' % (tt, unit, n_bins))
+
+            
 
 def pad_matrices_with_zeros(x1_min, x1_max, pp1, dpp1, x2_min, x2_max, pp2, dpp2, tolerance=1e-2):
     '''
@@ -564,6 +616,7 @@ def make_tphase(freq,  min_en=3., max_en=70., t_step=1000, n_bins=32, orbitfile=
                                            n_bins, freq, nudot, t_step, min_en, max_en))
 
             run()
+            shutil.copy('%s%s_TPHASE.fits' % (tt, unit), '%s%s_TPHASE_%03d.fits' % (tt, unit, n_bins))
 
 def rebin_matrix(e_min, e_max, pp_input, dpp_input, min_s_n = 50, only_pulsed=False):
     '''
@@ -697,11 +750,200 @@ def pulse_fraction_from_data_rms(counts, counts_err, n_harm=3):
 #               2. the fit results to be plotted in a file
 #               3. Figure in pdf
 
+def get_pulsed_fraction(e_min, e_max, pp, dpp, method_calc_rms='adaptive', output_file=None, force_recompute=False):
 
+    if output_file is not None:
+        filename = output_file
+        if os.path.isfile(filename) and (force_recompute is False):
+            ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac = np.loadtxt(filename, unpack=True, skiprows=1)
+            return ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac
+
+    ee_pulsed=(e_max+e_min)/2.
+    dee_pulsed=(e_max-e_min)/2.
+    pulsed_frac=np.zeros(len(e_min))
+    dpulsed_frac=np.zeros(len(e_min))
+
+    old_n_harm = pp.shape[1]
+
+    for i in range(0, len(e_min)):
+        x = pp[i,:]
+        dx = dpp[i,:]
+        if method_calc_rms=='minmax':
+        #Min/MAX 
+            pulsed_frac[i], dpulsed_frac[i] = pulse_fraction_from_data_min_max(x,dx)
+        elif method_calc_rms=='fixednharm':
+        #RMS with fixed n_harm
+            pulsed_frac[i] = pulse_fraction_from_data_rms(x,dx, n_harm=3)
+            dpulsed_frac[i] = get_error_from_simul(x,dx, pulse_fraction_from_data_rms, n_harm=3)
+        else:
+        #RMS with adaptive n_harm
+            print(e_min[i], e_max[i])
+            pulsed_frac[i] = fft_pulsed_fraction(x, dx, level=0.1, n_harm_min=2, n_harm_max=8, plot=False)
+            dpulsed_frac[i] = get_error_from_simul(x,dx, fft_pulsed_fraction, level=0.1, n_harm_min=2, 
+                                                         n_harm_max=8, verbose=False,  n_simul=300)
+        
+    if output_file is not None:
+        with open(output_file, 'w') as ff:
+            ff.write("Energy Energy_error Pullsed_fraction Pulsed_fraction_Error\n")
+            for i in range(0, len(e_min)):
+                ff.write("%.2f %.2f %.4f %.4f\n" % (ee_pulsed[i], dee_pulsed[i], 
+                            pulsed_frac[i], dpulsed_frac[i]))
+    return ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac
+
+def elaborate_pulsed_fraction(ee_pulsed, dee_pulsed, pulsed_frac, dpulsed_frac, debug_plots=True, e1=10, e2=20, 
+                              stem='', poly_deg=3, title='', save_plot=True, e_threshold=0.3, 
+                              division_derivative_order=2):
+    from scipy import interpolate
+    smoothing = interpolate.UnivariateSpline(ee_pulsed, pulsed_frac, w=1./dpulsed_frac, k=3)
+    fake_en = np.linspace(ee_pulsed[0], ee_pulsed[-1], 1000)
+    smoothed_pulsed_frac_deriv=smoothing.derivative(1)(fake_en)
+    smoothed_pulsed_frac_deriv_2=smoothing.derivative(2)(fake_en)
+
+    def get_zeros(energy,deriv,deriv_2,selection):
+        positive = deriv[selection] > 0
+        ind_extreme = np.where(np.bitwise_xor(positive[1:], positive[:-1]))
+        #print(energy[selection][ind_extreme])
+        ind_zeros = deriv_2[selection][ind_extreme] > 0
+        #print(deriv_2[selection][ind_extreme])
+        #print(ind_zeros)
+        return energy[selection][ind_extreme][ind_zeros]
+
+    def get_minimum(energy, deriv, selection):
+        ind_min = np.argmin(deriv[selection])
+        return energy[selection][ind_min]
+
+    if debug_plots:
+        fig,axes = plt.subplots(1,2,sharex=True)
+        axes[0].errorbar(ee_pulsed,pulsed_frac, xerr=dee_pulsed, yerr=dpulsed_frac, linestyle='')
+        axes[0].set_xlabel('Energy [keV]')
+        axes[0].set_ylabel('Pulsed Fraction')
+        axes[0].plot(fake_en, smoothing(fake_en))
+
+        pulsed_frac_deriv=np.gradient(pulsed_frac, ee_pulsed)
+        axes[1].plot(ee_pulsed, (pulsed_frac_deriv))
+        axes[1].plot(fake_en, smoothed_pulsed_frac_deriv)
+        axes[1].set_xlabel('Energy [keV]')
+        axes[1].set_ylabel('Pulsed Fraction Derivative')
+
+    ind_range1 = (fake_en >= e1) & (fake_en <=e2)
+
+    if division_derivative_order == 2:
+        e_turn = get_zeros(fake_en, smoothed_pulsed_frac_deriv_2, None, ind_range1)
+    else:
+        e_turn = get_zeros(fake_en, smoothed_pulsed_frac_deriv, smoothed_pulsed_frac_deriv_2, ind_range1)
+    
+    if len(e_turn) == 0:
+        logger.info('Using the minimum derivative instead of zero in dividing regimes')
+        e_turn = get_minimum(fake_en, smoothed_pulsed_frac_deriv, ind_range1)
+    else:
+        logger.info('Getting a zero of derivative to divide regimes')
+        e_turn = e_turn[0]
+
+    logger.info("We separate PF fit at %f keV" % e_turn)
+
+    if debug_plots:
+        axes[0].axvline(e_turn, 0, 1, color='cyan')
+        axes[1].axvline(e_turn, 0, 1, color='cyan')
+
+    ind_low = ee_pulsed <= e_turn
+    ind_high = ee_pulsed > e_turn
+    logger.info('We use %d points in the low-energy' % np.sum(ind_low))
+
+    pulsed_fit_low = fit_pulsed_frac(ee_pulsed[ind_low],dee_pulsed[ind_low], 
+                                        pulsed_frac[ind_low], dpulsed_frac[ind_low], n_gauss=1, 
+                                       center=[[6.5, 5.5, 7.5]],
+                                        sigma=[[0.2,0,1.0]],
+                                        amplitude=[[0,-1,1]],
+                                       degree_pol=poly_deg, plot_final=True, stem=stem+'low')
+
+    e_cyc = get_zeros(fake_en, smoothed_pulsed_frac_deriv, smoothed_pulsed_frac_deriv_2, fake_en>e_turn)
+    
+    if len(e_cyc)>1:
+        logger.info("Checking for too close Gaussians in %d elements" % len(e_cyc))
+        diff_e_cyc= ( 1- e_cyc[0:-1]/e_cyc[1:] )
+        ind_stay = diff_e_cyc > e_threshold
+#         print(e_cyc)
+#         print(diff_e_cyc)
+#         print(ind_stay)
+        e_cyc = [e_cyc[0]] + [ee for ee in e_cyc[1:][ind_stay]]
+        logger.info("Kept %d gaussians" % len(e_cyc))
+    
+    n_gauss = len(e_cyc)
+    if n_gauss >0:
+        center = [ [ee, (1-e_threshold)*ee, (1+e_threshold)*ee] for ee in e_cyc]
+        sigma=[[1+0*ee,0,3] for ee in e_cyc]
+        amplitude=[[0*ee,-2,2] for ee in e_cyc ]
+    else:
+        center = []
+        sigma = []
+        amplitude = []
+        
+    #print(center,sigma,amplitude)
+    logger.info('We use %d points in the high-energy' % np.sum(ind_high))
+    high_poly_deg = poly_deg
+    
+    if np.sum(ind_high) <= high_poly_deg + n_gauss * 3:
+        logger.info("Reducing polynomial order")
+        while (np.sum(ind_high) <= high_poly_deg + n_gauss * 3) and high_poly_deg >0:
+            
+            high_poly_deg -= 1
+        
+        if (np.sum(ind_high) <= high_poly_deg + n_gauss * 3):
+            n_gauss = 0
+            center = []
+            sigma = []
+            amplitude = []
+            high_poly_deg=1
+    
+    logger.info('We fit an order %d polynomial in the high-energy section' % high_poly_deg)
+    logger.info('We fit %d gaussians in the high-energy section' % n_gauss)
+    
+    pulsed_fit_high = fit_pulsed_frac(ee_pulsed[ind_high],dee_pulsed[ind_high], 
+                                        pulsed_frac[ind_high], dpulsed_frac[ind_high], n_gauss=n_gauss, 
+                                       center=center,
+                                        sigma=sigma,
+                                        amplitude=amplitude,
+                                       degree_pol=high_poly_deg, plot_final=True, stem=stem+'high')
+
+    
+    from matplotlib.pyplot import cm
+    comps_low = pulsed_fit_low.eval_components(x=ee_pulsed[ind_low])
+    comps_high = pulsed_fit_high.eval_components(x=ee_pulsed[ind_high])
+    bb_low = (pulsed_frac[ind_low] - pulsed_fit_low.best_fit) / dpulsed_frac[ind_low]
+    bb_high = (pulsed_frac[ind_high] - pulsed_fit_high.best_fit) / dpulsed_frac[ind_high]
+
+    col = cm.viridis(np.linspace(0, 1, 6))
+
+    fig, (ax1, ax2) = plt.subplots(2, 1, figsize=(6.2, 6.0), sharex=True, gridspec_kw={'height_ratios': [3, 1],
+                                                                        'hspace': 0.0}
+                                )
+    ax1.errorbar(ee_pulsed,pulsed_frac, xerr=dee_pulsed, yerr=dpulsed_frac, linestyle=''
+                , fmt='.', color=col[4], label='data')
+    ax1.plot(ee_pulsed[ind_low], pulsed_fit_low.best_fit, '-', label='best fit (low)', color=col[0])
+    ax1.plot(ee_pulsed[ind_low], comps_low['poly_'], '--', label='Polynomial (low)', color=col[1])
+    ax1.plot(ee_pulsed[ind_high], pulsed_fit_high.best_fit, '-', label='best fit (high)', color=col[2])
+    ax1.plot(ee_pulsed[ind_high], comps_high['poly_'], '--', label='Polynomial (high)', color=col[3])
+
+    ax1.legend(loc='upper left')
+    ax1.set_xscale('log')
+    ax2.set_xlabel('E (keV)')
+    ax1.set_ylabel('PF')
+    # ax1.set_ylim(ylim)
+    # ax1.set_xlim(xlim)
+    ax1.set_title(title)
+
+    ax2.set_ylabel('Residuals')
+    ax1.set_ylim(-0.1, 1)
+    ax2.errorbar(ee_pulsed, np.concatenate([bb_low,bb_high]), xerr=dee_pulsed, yerr=1., fmt='.', color=col[2])
+    ax2.axhline(y=0, color='k', linestyle='--')
+    if save_plot:
+        plt.savefig(stem + 'global_pulsed_fitted.pdf')
+
+    return pulsed_fit_low, pulsed_fit_high
 
 def fit_pulsed_frac(en, den, pf, dpf, stem=None, degree_pol=4, n_gauss=0, center=[],
                     sigma=[], amplitude=[],
-                    plot_final=True, print_results=True):
+                    plot_final=True, print_results=True, ylim=[0,1], xlim=[3,70]):
     '''
     this function will fit an input energy range of the pulse profile.
     the fitting function will be a simple polynomial + gaussian
@@ -729,7 +971,8 @@ def fit_pulsed_frac(en, den, pf, dpf, stem=None, degree_pol=4, n_gauss=0, center
     from lmfit.models import PolynomialModel, GaussianModel
 
     if len(center) != n_gauss or len(sigma) != n_gauss or len (amplitude) != n_gauss:
-        logger.error("You should provide centers, sigmas, and amplitudes for %d gaussians" % n_gauss)
+        logger.error("You provided %d centers, %d sigmas, and %d amplitudes for %d gaussians" % (len(center), 
+                            len(sigma), len(amplitude), n_gauss))
         return
 
     col = cm.viridis(np.linspace(0, 1, 6))
@@ -771,6 +1014,9 @@ def fit_pulsed_frac(en, den, pf, dpf, stem=None, degree_pol=4, n_gauss=0, center
         axes[0].legend(loc='upper left')
         axes[0].set_xlabel('E [keV]')
         axes[0].set_ylabel('PF')
+        axes[0].set_ylim(ylim)
+        axes[0].set_xlim(xlim)
+        
 
         axes[1].errorbar(en, pf, xerr=den, yerr=dpf, fmt='.', color=col[0])
         for N in range(1, n_gauss+1):
@@ -779,6 +1025,9 @@ def fit_pulsed_frac(en, den, pf, dpf, stem=None, degree_pol=4, n_gauss=0, center
         axes[1].legend(loc='upper left')
         axes[1].set_xlabel('E [keV]')
         axes[1].set_ylabel('PF')
+        axes[1].set_ylim(ylim)
+        axes[1].set_xlim(xlim)
+        
         plt.savefig(stem + 'fit_results.pdf')
 
     if print_results:
@@ -805,6 +1054,9 @@ def fit_pulsed_frac(en, den, pf, dpf, stem=None, degree_pol=4, n_gauss=0, center
         ax1.set_xscale('log')
         ax2.set_xlabel('E (keV)')
         ax1.set_ylabel('PF')
+        ax1.set_ylim(ylim)
+        ax1.set_xlim(xlim)
+        
         ax2.set_ylabel('Residuals')
         ax1.set_ylim(-0.1, 1)
         ax2.errorbar(en, bb, xerr=den, yerr=1., fmt='.', color=col[2])
@@ -1101,7 +1353,7 @@ def get_fourier_coeff_error( counts, counts_err, n_simul=100, use_poisson=False)
     return np.std(sim_phi0), np.std(sim_phi0_2,), np.std(sim_A), np.std(sim_A2)
 
 def rate_filter(time, rate, drate, level_down=0, level_up = 1e10, quantile=0.8, make_plot=True, 
-                far=1e-4, rate_label='rate'):
+                far=1e-4, rate_label='rate', histo_bins=300):
     '''
 rate_filter(self, typical_level=0.5, quantile=0.7,  make_plot=True, far=1e-3,
                     tmin=0,tmax=1e12):
@@ -1120,7 +1372,7 @@ The filter is controlled in the following way:
         axes[0].set_ylabel(rate_label)
         #axes[0].set_title(self.target + ' ' + self.obs_id)
         axes[0].set_xlabel('Time [s]')
-        n_hist, edges, patches = axes[1].hist(rate, bins=300, density=True, facecolor='green',
+        n_hist, edges, patches = axes[1].hist(rate, bins=histo_bins, density=True, facecolor='green',
                                               alpha=0.75)
         axes[1].set_xlabel(rate_label)
 
@@ -1193,6 +1445,7 @@ def write_orbit_file_from_gbm_page(url, file_name='orbit.dat',
         logger.warning('Not able to retrieve the orbital parameters as %s could not be read' % orbital_parameters[0])
         return None
     # it writes the orbit file
+    logger.info('Writing the following orbit parameters')
     with open(file_name, 'w') as ff:
         for kk in orbital_parameters:
             if 'T' in kk:
@@ -1207,6 +1460,7 @@ def write_orbit_file_from_gbm_page(url, file_name='orbit.dat',
                     ff.write('%f\n' % (t_90.mjd - 0.25 * float(orbit['Orbital Period'])))
             else:
                 ff.write(orbit[kk] + '\n')
+            logger.info('%s %s' % (kk , orbit[kk]))
     return file_name
 
 def get_cross_correlation(pp, plot=False):
